/* global Bdf,BdfApi,Fql,$$,Overlay */
/**
 * Objet global définissant l'espace de nom Multi
 * 
 * @namespace Index
 */
var Index = {};

Index.ARGS = {
    sphereName: "",
    title: "",
    withLogo: false,
    centralUser: {
        sphere: "",
        login: "",
        name: "",
        lang: "",
        locale: ""
    }
};
Index.URL = "";
Index.SORT_ARRAY = [
	{
		name: "creationdate-desc",
		locKey: "_ label.selection.sorttype_creationdate"
	},
	{
		name: "titre-asc",
		locKey: "_ label.selection.sorttype_titre"
	},
	{
		name: "id-asc",
		locKey: "_ label.selection.sorttype_id"
	}
];
Index.SCOPE_ARRAY = [
	{
		name: "titre",
		locKey: "_ label.selection.scope_titre"
	},
	{
		name: "entete",
		locKey: "_ label.selection.scope_entete"
	},
	{
		name: "fiche",
		locKey: "_ label.selection.scope_fiche"
	}
];
Index.FICHOTHEQUE_MAP = new Map();

Index.init = function (args) {
    Index.URL = "index-" + args.sphereName;
    $$("layout").html(Bdf.render("index:layout", {
        title: args.title,
        withLogo: args.withLogo,
        centralUser: args.centralUser,
        contentScopeArray: Index.toOptionArray(Index.SCOPE_ARRAY, "titre"),
	sortTypeArray: Index.toOptionArray(Index.SORT_ARRAY, "creationdate-desc")
    }));
    Index.initActions();
    $.ajax({
        url: Index.URL,
        data: {
           json: "presence"
        },
        xhrFields: {
           withCredentials: true
        },
        dataType: Bdf.jsonDataType,
        success: function (data, textStatus) {
            Bdf.checkData(data, "presenceMap", function (presenceMap) {
                _listPresence(presenceMap);
            });
        }
    });
    
    function _listPresence(presenceMap) {
        var array = new Array();
        for(let prop in presenceMap) {
            let fichotheque = presenceMap[prop];
            Index.FICHOTHEQUE_MAP.set(prop, fichotheque);
            array.push(toLink(fichotheque));
        }
        $$("presence").html(Bdf.render("index:presence", {
            list: array
        }));
    }
    
    function toLink(fichothequeInfo) {
        var text;
        if (fichothequeInfo.title) {
            text = fichothequeInfo.title;
        } else {
            text = fichothequeInfo.name;
        }
        var title = "";
        if (fichothequeInfo.phrases.long) {
            title = fichothequeInfo.phrases.long;
        }
        return {
            name: fichothequeInfo.name,
            href: fichothequeInfo.name + "/session",
            text: text,
            title: title,
            init: fichothequeInfo.init
        };
    }
    
};

Index.initActions = function () {
    $$("button_exit").click(function () {
        $.ajax({
            url: Index.URL,
            data: {
               cmd: "exit",
               json: "ping"
            },
            xhrFields: {
               withCredentials: true
            },
            dataType: Bdf.jsonDataType,
            success: function (data, textStatus) {
                window.location.reload();
            }
        });
    });
    $$("form_search").submit(function(event) {
        event.preventDefault();
        let q = this["q"].value;
        if (q) {
            q = q.trim();
        }
        if (q) {
            let scopeSelect = this["scope"];
            let contentScope = scopeSelect.options[scopeSelect.selectedIndex].value;
            let sortSelect = this["sort"];
            let sortType = sortSelect.options[sortSelect.selectedIndex].value;
            Index.showSearch(q, contentScope, sortType);
        }
    });
};

Index.showSearch = function (q, contentScope, sortType) {
    var formId = Bdf.generateId();
    var targetArray = new Array();
    for(let fichotheque of Index.FICHOTHEQUE_MAP.values()) {
        let title = fichotheque.phrases.long;
        if (!title) {
            title = fichotheque.title;
        }
        targetArray.push({
            name: fichotheque.name,
            title: title,
            queries: [{}],
            fichotheque: fichotheque
        });
    }
    var max = targetArray.length;
    var count = 0;
    var contentScopeArray = Index.toOptionArray(Index.SCOPE_ARRAY, contentScope);
    var sortTypeArray = Index.toOptionArray(Index.SORT_ARRAY, sortType);
    Overlay.start({
        content: Bdf.render("index:overlay-search", {
            q: q,
            formId: formId,
            targetArray: targetArray,
            contentScopeArray: contentScopeArray,
            sortTypeArray: sortTypeArray
        }),
        isWaiting: true,
        supplementaryClasses: {
            content: "index-search-OverlayContent"
        },
        afterStart: function (overlayId, waitingCallback) {
            $$(formId, "search").submit(function(event) {
                event.preventDefault();
                let newQ = this["q"].value;
                if (newQ) {
                        newQ = newQ.trim();
                }
                if (newQ) {
                    count = 0;
                    q = newQ;
                    let scopeSelect = this["scope"];
                    contentScope = scopeSelect.options[scopeSelect.selectedIndex].value;
                    let sortSelect = this["sort"];
                    sortType = sortSelect.options[sortSelect.selectedIndex].value;
                    $$(formId, "button_submit").prop("disabled", true);
                    for(let target of targetArray) {
                        $$(formId, "target", target.name).html("");
                        $$(formId, "fichecount", target.name).text("");
                       _search(target, waitingCallback);
                    }
                }
            });
            for(let target of targetArray) {
                _search(target, waitingCallback);
            }
        }
    });
    
    function _search(target, waitingCallback) {
        var ficheQueryArray = new Array();
        for(let query of target.queries) {
            let fq = Object.assign({
                content: {
                    scope: contentScope,
                    q: q,
                    type: "simple"
                }
            }, query);
            ficheQueryArray.push(new Fql.FicheQuery(fq));
        }
        BdfApi.Ajax.loadFiches(new BdfApi.Config(target.name + "/"), ficheQueryArray, {
            callback: function (data) {
                count++;
                if (count === max) {
					$$(formId, "button_submit").prop("disabled", false);
                    waitingCallback();
                }
                $$(formId, "fichecount", target.name).text("(" + data.fiches.count + ")");
                let $target = $$(formId, "target", target.name);
                $target.html(Bdf.render("index:results", {
                    name: target.name,
                    fiches: data.fiches
                }));
                Bdf.$($target, {role: "fiche"}).click(_showFiche);
            },
            requestParameters: {
                sort: sortType
            }
        });
    }    

    function _showFiche() {
        let link = this;
        $$(formId, "button_blank").attr("href", link.href).removeClass("hidden");
        $$(formId, "button_odt").attr("href", link.dataset.fichotheque + "/fiches/" + link.dataset.corpus + "-" + link.dataset.id + ".odt").removeClass("hidden");
        if (link.dataset.editable) {
            $$(formId, "button_edit").attr("href", link.dataset.fichotheque + "/edition?page=fiche-change&page-result-options=standalone&corpus=" + link.dataset.corpus + "&id=" + link.dataset.id).removeClass("hidden");
        } else {
            $$(formId, "button_edit").addClass("hidden");
        }
    }
    
};


Index.toOptionArray= function (array, selectedValue) {
    let optionArray = new Array();
    for(let option of array) {
        let selected = (option.name === selectedValue);
        optionArray.push({
                value: option.name,
                locKey: option.locKey,
                selected: selected
        });
    }
    return optionArray;
};

$(function(){
    Bdf.initTemplates();
    Index.init(Index.ARGS);
});
